@@ -1,13 +1,19 @@
+import pipes
+from functools import wraps, partial
+from io import TextIOWrapper
 from itertools import chain
-from subprocess import Popen as Popen, STDOUT, PIPE
+from subprocess import STDOUT, PIPE
 from tempfile import TemporaryFile
-from testfixtures.compat import basestring, PY3, zip_longest
+from testfixtures.compat import basestring, PY3, zip_longest, reduce, PY2
 from testfixtures.utils import extend_docstring
 
-try:
-    from unittest.mock import Mock
-except ImportError:
-    from mock import Mock
+from .mock import Mock, call
+
+
+def shell_join(command):
+    if not isinstance(command, basestring):
+        command = " ".join(pipes.quote(part) for part in command)
+    return command
 
 
 class PopenBehaviour(object):
@@ -25,106 +31,58 @@ class PopenBehaviour(object):
         self.poll_count = poll_count
 
 
-class MockPopen(object):
+def record(func):
+    @wraps(func)
+    def recorder(self, *args, **kw):
+        self._record((func.__name__,), *args, **kw)
+        return func(self, *args, **kw)
+    return recorder
+
+
+class MockPopenInstance(object):
     """
-    A specialised mock for testing use of :class:`subprocess.Popen`.
-    An instance of this class can be used in place of the
-    :class:`subprocess.Popen` and is often inserted where it's needed using
-    :func:`mock.patch` or a :class:`Replacer`.
+    A mock process as returned by :class:`MockPopen`.
     """
 
-    default_behaviour = None
+    #: A :class:`~unittest.mock.Mock` representing the pipe into this process.
+    #: This is only set if ``stdin=PIPE`` is passed the constructor.
+    #: The mock records writes and closes in :attr:`MockPopen.all_calls`.
+    stdin = None
 
-    def __init__(self):
-        self.commands = {}
-        self.mock = mock = Mock()
-        self.mock.Popen.side_effect = self.Popen
-        mock.Popen_instance = Mock(spec=Popen)
-        inst = mock.Popen.return_value = mock.Popen_instance
-        inst.communicate.side_effect = self.communicate
-        inst.wait.side_effect = self.wait
-        inst.send_signal.side_effect = self.send_signal
-        inst.terminate.side_effect = self.terminate
-        inst.kill.side_effect = self.kill
-        inst.poll.side_effect = self.poll
-        if PY3:
-            def __enter__(self):
-                return inst
-            inst.__enter__ = __enter__
+    #: A file representing standard output from this process.
+    stdout = None
 
-            def __exit__(self, exc_type, exc_val, exc_tb):
-                inst.wait()
-                for stream in inst.stdout, inst.stderr:
-                    if stream:
-                        stream.close()
+    #: A file representing error output from this process.
+    stderr = None
 
-            inst.__exit__ = __exit__
+    def __init__(self, mock_class, root_call,
+                 args, bufsize=0, executable=None,
+                 stdin=None, stdout=None, stderr=None,
+                 preexec_fn=None, close_fds=False, shell=False, cwd=None,
+                 env=None, universal_newlines=False,
+                 startupinfo=None, creationflags=0, restore_signals=True,
+                 start_new_session=False, pass_fds=(),
+                 encoding=None, errors=None, text=None):
+        self.mock = Mock()
+        self.class_instance_mock = mock_class.mock.Popen_instance
+        #: A :func:`unittest.mock.call` representing the call made to instantiate
+        #: this mock process.
+        self.root_call = root_call
+        #: The calls made on this mock process, represented using
+        #: :func:`~unittest.mock.call` instances.
+        self.calls = []
+        self.all_calls = mock_class.all_calls
 
-    def _resolve_behaviour(self, stdout, stderr, returncode,
-                           pid, poll_count, behaviour):
-        if behaviour is None:
-            return PopenBehaviour(
-                stdout, stderr, returncode, pid, poll_count
-            )
-        else:
-            return behaviour
+        cmd = shell_join(args)
 
-    def set_command(self, command, stdout=b'', stderr=b'', returncode=0,
-                    pid=1234, poll_count=3, behaviour=None):
-        """
-        Set the behaviour of this mock when it is used to simulate the
-        specified command.
-
-        :param command: A string representing the command to be simulated.
-
-        If supplied, ``behaviour`` must be either a :class:`PopenBehaviour`
-        instance or a callable that takes the ``command`` string representing
-        the command to be simulated and the ``stdin`` for that command and
-        returns a :class:`PopenBehaviour` instance.
-        """
-        self.commands[command] = self._resolve_behaviour(
-            stdout, stderr, returncode, pid, poll_count, behaviour
-        )
-
-    def set_default(self, stdout=b'', stderr=b'', returncode=0,
-                    pid=1234, poll_count=3, behaviour=None):
-        """
-        Set the behaviour of this mock when it is used to simulate commands
-        that have no explicit behavior specified using
-        :meth:`~MockPopen.set_command` or :meth:`~MockPopen.set_callable`.
-
-        If supplied, ``behaviour`` must be either a :class:`PopenBehaviour`
-        instance or a callable that takes the ``command`` string representing
-        the command to be simulated and the ``stdin`` for that command and
-        returns a :class:`PopenBehaviour` instance.
-        """
-        self.default_behaviour = self._resolve_behaviour(
-            stdout, stderr, returncode, pid, poll_count, behaviour
-        )
-
-    def __call__(self, *args, **kw):
-        return self.mock.Popen(*args, **kw)
-
-    def Popen(self, args, bufsize=0, executable=None,
-              stdin=None, stdout=None, stderr=None,
-              preexec_fn=None, close_fds=False, shell=False, cwd=None,
-              env=None, universal_newlines=False,
-              startupinfo=None, creationflags=0, restore_signals=True,
-              start_new_session=False, pass_fds=(), encoding=None, errors=None):
-
-        if isinstance(args, basestring):
-            cmd = args
-        else:
-            cmd = ' '.join(args)
-
-        behaviour = self.commands.get(cmd, self.default_behaviour)
+        behaviour = mock_class.commands.get(cmd, mock_class.default_behaviour)
         if behaviour is None:
             raise KeyError('Nothing specified for command %r' % cmd)
 
         if callable(behaviour):
             behaviour = behaviour(command=cmd, stdin=stdin)
 
-        self.returncode = behaviour.returncode
+        self.behaviour = behaviour
 
         stdout_value = behaviour.stdout
         stderr_value = behaviour.stderr
@@ -148,68 +106,152 @@ class MockPopen(object):
                 value.write(mock_value)
                 value.flush()
                 value.seek(0)
-            setattr(self.mock.Popen_instance, name, value)
+                if PY3 and (universal_newlines or text or encoding):
+                    value = TextIOWrapper(value, encoding=encoding, errors=errors)
+            setattr(self, name, value)
 
         if stdin == PIPE:
-            self.mock.Popen_instance.stdin = Mock()
+            self.stdin = Mock()
+            for method in 'write', 'close':
+                record_writes = partial(self._record, ('stdin', method))
+                getattr(self.stdin, method).side_effect = record_writes
 
-        self.mock.Popen_instance.pid = behaviour.pid
-        self.mock.Popen_instance.returncode = None
+        self.pid = behaviour.pid
+        #: The return code of this mock process.
+        self.returncode = None
         if PY3:
-            self.mock.Popen_instance.args = args
+            self.args = args
 
-        return self.mock.Popen_instance
+    def _record(self, names, *args, **kw):
+        for mock in self.class_instance_mock, self.mock:
+            reduce(getattr, names, mock)(*args, **kw)
+        for base_call, store in (
+            (call, self.calls),
+            (self.root_call, self.all_calls)
+        ):
+            store.append(reduce(getattr, names, base_call)(*args, **kw))
 
     if PY3:
+        def __enter__(self):
+            return self
+
+        def __exit__(self, exc_type, exc_val, exc_tb):
+            self.wait()
+            for stream in self.stdout, self.stderr:
+                if stream:
+                    stream.close()
+
+        @record
         def wait(self, timeout=None):
             "Simulate calls to :meth:`subprocess.Popen.wait`"
-            self.mock.Popen_instance.returncode = self.returncode
+            self.returncode = self.behaviour.returncode
             return self.returncode
 
+        @record
         def communicate(self, input=None, timeout=None):
             "Simulate calls to :meth:`subprocess.Popen.communicate`"
-            self.wait()
-            i = self.mock.Popen_instance
-            return (i.stdout and i.stdout.read(),
-                    i.stderr and i.stderr.read())
+            self.returncode = self.behaviour.returncode
+            return (self.stdout and self.stdout.read(),
+                    self.stderr and self.stderr.read())
     else:
+        @record
         def wait(self):
             "Simulate calls to :meth:`subprocess.Popen.wait`"
-            self.mock.Popen_instance.returncode = self.returncode
+            self.returncode = self.behaviour.returncode
             return self.returncode
 
+        @record
         def communicate(self, input=None):
             "Simulate calls to :meth:`subprocess.Popen.communicate`"
-            self.wait()
-            i = self.mock.Popen_instance
-            return (i.stdout and i.stdout.read(),
-                    i.stderr and i.stderr.read())
+            self.returncode = self.behaviour.returncode
+            return (self.stdout and self.stdout.read(),
+                    self.stderr and self.stderr.read())
 
+    @record
     def poll(self):
         "Simulate calls to :meth:`subprocess.Popen.poll`"
-        while self.poll_count and self.mock.Popen_instance.returncode is None:
+        while self.poll_count and self.returncode is None:
             self.poll_count -= 1
             return None
         # This call to wait() is NOT how poll() behaves in reality.
         # poll() NEVER sets the returncode.
         # The returncode is *only* ever set by process completion.
         # The following is an artifact of the fixture's implementation.
-        return self.wait()
+        self.returncode = self.behaviour.returncode
+        return self.returncode
 
-    # These are here to check parameter types
+    @record
     def send_signal(self, signal):
         "Simulate calls to :meth:`subprocess.Popen.send_signal`"
         pass
 
+    @record
     def terminate(self):
         "Simulate calls to :meth:`subprocess.Popen.terminate`"
         pass
 
+    @record
     def kill(self):
         "Simulate calls to :meth:`subprocess.Popen.kill`"
         pass
 
 
+class MockPopen(object):
+    """
+    A specialised mock for testing use of :class:`subprocess.Popen`.
+    An instance of this class can be used in place of the
+    :class:`subprocess.Popen` and is often inserted where it's needed using
+    :func:`unittest.mock.patch` or a :class:`~testfixtures.Replacer`.
+    """
+
+    default_behaviour = None
+
+    def __init__(self):
+        self.commands = {}
+        self.mock = Mock()
+        #: All calls made using this mock and the objects it returns, represented using
+        #: :func:`~unittest.mock.call` instances.
+        self.all_calls = []
+
+    def _resolve_behaviour(self, stdout, stderr, returncode,
+                           pid, poll_count, behaviour):
+        if behaviour is None:
+            return PopenBehaviour(
+                stdout, stderr, returncode, pid, poll_count
+            )
+        else:
+            return behaviour
+
+    def set_command(self, command, stdout=b'', stderr=b'', returncode=0,
+                    pid=1234, poll_count=3, behaviour=None):
+        """
+        Set the behaviour of this mock when it is used to simulate the
+        specified command.
+
+        :param command: A string representing the command to be simulated.
+        """
+        self.commands[shell_join(command)] = self._resolve_behaviour(
+            stdout, stderr, returncode, pid, poll_count, behaviour
+        )
+
+    def set_default(self, stdout=b'', stderr=b'', returncode=0,
+                    pid=1234, poll_count=3, behaviour=None):
+        """
+        Set the behaviour of this mock when it is used to simulate commands
+        that have no explicit behavior specified using
+        :meth:`~MockPopen.set_command` or :meth:`~MockPopen.set_callable`.
+        """
+        self.default_behaviour = self._resolve_behaviour(
+            stdout, stderr, returncode, pid, poll_count, behaviour
+        )
+
+    def __call__(self, *args, **kw):
+        self.mock.Popen(*args, **kw)
+        root_call = call.Popen(*args, **kw)
+        self.all_calls.append(root_call)
+        return MockPopenInstance(self, root_call, *args, **kw)
+
+
 set_command_params = """
 :param stdout:
     A string representing the simulated content written by the process
@@ -227,6 +269,11 @@ set_command_params = """
     Specifies the number of times :meth:`MockPopen.poll` can be
     called before :attr:`MockPopen.returncode` is set and returned
     by :meth:`MockPopen.poll`.
+
+If supplied, ``behaviour`` must be either a :class:`PopenBehaviour`
+instance or a callable that takes the ``command`` string representing
+the command to be simulated and the ``stdin`` for that command and
+returns a :class:`PopenBehaviour` instance.
 """
 
 
